#' Function to determine best binning interval based on produced histograms
#'
#' This function allows you to identify which binning interval is better
#' based on the ammount of bins added by the Import funtion.
#' 
#' The output of this function is a figure with four histograms,
#' one for each possible binning (1, 2, 3, or 4 hours).
#' 
#' Required packages:
#'    CATkit
#'    
#' @param TimeCol Column of table where Datetime values are stored. Defaults to 1.
#' @param firstvarcol Column of table where the first variable to analyze is stored. Defaults to 2.
#' @param lastvarcol Column of table where the last variable to analyze is stored. Defaults to 6.
#' @param maxGap Maximum gap of missing values allowed in the data set. In this stage we WANT to know how many missing values are in the data for each binning interval.  Gap is set very high. Defaults to 30.
#' @keywords determine binning
#' @export
#' @examples
#' determine_binning(TimeCol=1,firstvarcol=2,lastvarcol=6,maxGap=30)

determine_binning <- function(TimeCol=1,firstvarcol=2,lastvarcol=6,maxGap=30){
  
  ###This function allows you to identify which binning interval is better
  ###based on the amount of bins added by the Import function.
  
  ###The output of this function is a figure with four histograms,
  ###one for each possible binning (1, 2, 3, or 4 hours).
  
  ###Required packages:
  ## CatKit
  
  missingvalues <- c()
  
  for(i in recordnamelist){
    filename <- paste(sub(pattern=".csv",replacement="",x=i),"_formatted_avg-dup.csv",sep="")
    
    fileName <- file.path(workdir,filename)
    
    myHeader <- read.table(fileName, header = TRUE, nrows = 1,sep = ",", skip = 0)
    
    
    myData1A <- read.table(fileName, header = TRUE, sep = ",",skip = 0, stringsAsFactors = FALSE, comment.char = "",col.names = names(myHeader))
    
    myData1 <- myData1A[, c(TimeCol = TimeCol, valCols = seq(from=firstvarcol,to=lastvarcol,by=1))]
    
    myData1[, TimeCol[1]] <- as.POSIXct(strptime(x = paste(myData1[,TimeCol[1]], myData1[, TimeCol[1]]), format = "%Y%m%d%H%M" , tz = "GMT"))
    
    
    for(ii in c(60,120,180,240)){
      
      myList <- do.call(Import, list(x = myData1, tsCol = TimeCol, valCols = seq(from=firstvarcol,to=lastvarcol,by=1), sumCols = 1,
                                     rowDurationSeconds = (ii*60), maxGap = maxGap, 
                                     verbose = 0, tz = "GMT"))
      
      addedbins <- length(myList$df[,1][!myList$df[,1] %in% myData1[,1]])
      #addedbins <- sum((myList$df[,1] %in% myData1[,1])==FALSE)
      missingvalues <- append(missingvalues,addedbins,after=length(missingvalues))
      
    }
  }
  
  
  
  allmissingvalues <- matrix(data=missingvalues,nrow=4,byrow=FALSE)
  
  
  if(.Platform$OS.type=="windows"){
    windows()
  }else{
    quartz()
  }
  
  par(mar=c(4.1,4.1,4.1,.5),mfrow=c(2,2))
  
  for(i in 1:nrow(allmissingvalues)){
    hist(allmissingvalues[i,],main=paste("Binning to",i,"hour(s)"),xlab="Amount of Missing Values",xlim=c(0,30))
  }
  
  
  
  
}

