#' Function to upload all medical record files in .csv format
#' in the directory in the R global environment.
#'
#' Required packages:
#'   -plyr
#'   -rapport
#'   -rapportools
#'    
#' The process occurs in the following order:

#' User chooses the working directory. (Where the files are located.)
#' The working directory is set.
#' A list with all files containing '.csv' is created 'recordnamelist'.
#' ******Make sure only files intended for analysis are in the directory.
#' Multiple data.frames are created, one for each file in recordnamelist.
#' Dates and time are formatted to YYYYMMDDHHmm format
#' Numerical and categorical data is separated.
#' i.e: temperature and way it was taken
#' NAs are introduced wherever empty values are found.
#' Data is rearranged in chronological order.
#' Time is rounded to nearest hour.
#' Duplicates are averaged
#'  
#' As an output, two CSV files are saved in the working directory.
#' One contains all data previous to averaging of duplicates.
#' this includes the categorical data for each vital sign.
#' The second, contains only numerical data after averaging duplicates.
#' CSV file names end in "_formatted.csv" and
#' "_formatted_avg-dup.csv", respectively.
#'    
#' *These functions are loaded only when they are needed and then unloaded.
#'  They mask the base 'mean' function which is used in the codes.
#'
#' @param 
#' @keywords load records
#' @export
#' @examples
#' record_load()
#' 


record_load <- function(){
  
  ###This function allows you to upload all medical record files in .csv format
  ###in a directory into the R global environment.
  
  ###required packages:
  ## plyr
  ## rapport
  ## rapportools
  
  ###The process occurs in the following order:
  
  ###User chooses the working directory. (Where the files are located.)
  ###The working directory is set.
  ###A list with all files containing '.csv' is created 'recordnamelist'.
  ###******Make sure only files intended for analysis are in the directory.
  ###Multiple data.frames are created, one for each file in recordnamelist.
  ###Dates and time are formatted to YYYYMMDDHHmm format
  ###Numerical and categorical data is separated.
  ###i.e: temperature and way it was taken
  ###NAs are introduced wherever empty values are found.
  ###Data is rearranged in chronological order.
  ###Time is rounded to nearest hour.
  ###Duplicates are averaged
  
  ###As an output, two CSV files are saved in the working directory.
  ###One contains all data previous to averaging of duplicates.
  ###this includes the categorical data for each vital sign.
  ###The second, contains only numerical data after averaging duplicates.
  ###CSV file names end in "_formatted.csv" and
  ###"_formatted_avg-dup.csv", respectively.
  
  #lockEnvironment(.GlobalEnv,FALSE)
  
  print("Set your Working Directory")
  Sys.sleep(1)
  
  workdir <<- dirname(file.choose())
  setwd(workdir)
  
  #make list of all files in working directory that have pattern ".csv"
  recordnamelist <<- list.files(pattern="*.csv")
  
  
  #make data frames of every file obtained in 'recordnamelist'
  for(i in 1:length(recordnamelist)){
    assign(recordnamelist[i],read.csv(recordnamelist[i]),envir=.GlobalEnv)
  }
  
  
  
  
  for(i in recordnamelist){
    require(rapport)
    require(rapportools)
    tempfile <- get(i)
    tempfile <- na.omit(tempfile)
    tempfile$newdate <- strptime(as.character(tempfile$X), "%b-%d-%Y")
    tempfile$newdate <- gsub("[[:punct:]]","",tempfile$newdate)
    tempfile$zeros <- rep("0000",nrow(tempfile))
    tempfile$date = paste(tempfile$newdate, tempfile$zeros, sep="")
    tempfile$Date.time <- as.numeric(tempfile$Date.time)
    tempfile$date <- as.numeric(tempfile$date)
    Datetime <- tempfile$date + tempfile$Date.time
    
    tempfile <- cbind(Datetime,tempfile)
    rm(Datetime)
    tempfile$newdate <- NULL
    tempfile$zeros <- NULL
    tempfile$date <- NULL
    tempfile[,3] <- NULL
    tempfile[,2] <- NULL
    
    
    tempfile[,7:11] <- tempfile[,2:6]
    
    tempfile[,2] <- gsub('[^.0-9]', '', tempfile[,2])
    tempfile[,3] <- gsub('[^.0-9]', '', tempfile[,3])
    tempfile[,4] <- gsub('[^.0-9]', '', tempfile[,4])
    tempfile[,5] <- gsub('[^.0-9]', '', tempfile[,5])
    tempfile[,6] <- gsub('[^.0-9]', '', tempfile[,6])
    
    tempfile[,7] <- gsub('[^A-Z]', '', tempfile[,7])
    tempfile[,8] <- gsub('[^A-Z]', '', tempfile[,8])
    tempfile[,9] <- gsub('[^A-Z]', '', tempfile[,9])
    tempfile[,10] <- gsub('[^A-Z]', '', tempfile[,10])
    tempfile[,11] <- gsub('[^A-Z]', '', tempfile[,11])
    
    tempfile[,2] <- as.numeric(tempfile[,2])
    tempfile[,3] <- as.numeric(tempfile[,3])
    tempfile[,4] <- as.numeric(tempfile[,4])
    tempfile[,5] <- as.numeric(tempfile[,5])
    tempfile[,6] <- as.numeric(tempfile[,6])
    
    
    
    for(j in 7:11){
      for(q in 1:nrow(tempfile)){
        if(is.empty(tempfile[q,j])==TRUE){
          tempfile[q,j] = NA
        }
      }
    }
    
    
    tempfile <- tempfile[order(tempfile[,1]),]
    
    assign(i,tempfile,envir=.GlobalEnv)
    
    
    unloadNamespace("rapport")
    unloadNamespace("rapportools")
  }
  
  for(i in recordnamelist){
    tempfile <- get(i)
  }
  
  for(i in recordnamelist){
    tempfile <- get(i)
    tempfile[,1] <- signif(tempfile[,1],digits=10)
    assign(i,tempfile,envir=.GlobalEnv)
    
    write.csv(tempfile,paste(sub(pattern=".csv",replacement="",x=i),"_formatted.csv",sep=""),row.names=FALSE)
  }
  
  
  for(i in recordnamelist){
    tempfile <- get(i)
    tempfile[,7:11] <- NULL
    columns <- colnames(tempfile)
    tempfile2 <- ddply(tempfile,.(tempfile$Datetime),colwise(mean,na.rm=TRUE,c(columns[2:6])))
    for(x in 2:ncol(tempfile2)){
      for(y in 1:nrow(tempfile2)){
        if(is.nan(tempfile2[y,x])==TRUE){
          tempfile2[y,x] <- NA
        }
      }
    }
    assign(i,tempfile2,envir=.GlobalEnv)
    write.csv(tempfile2,paste(sub(pattern=".csv",replacement="",x=i),"_formatted_avg-dup.csv",sep=""),row.names=FALSE)
    
  }
  
  length_files()
  
}
